require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/artifact/artifact_attributes'
require 'uu/os/artifact/artifact_create'
require 'uu/os/artifact/artifact_export'
require 'uu/os/artifact/artifact_get_export_data'
require 'uu/os/artifact/artifact_move'
require 'uu/os/artifact/artifact_set_attributes'
require 'uu/os/artifact/artifact_set_competent_role'
require 'uu/os/artifact/artifact_set_state'
require 'uu/os/artifact/artifact_state_type'
require 'uu/os/artifact/security_level'
require 'uu/os/artifact/artifact_get_reference_list'
require 'uu/os/artifact/artifact_set_visual_presentation'
require 'uu/os/artifact/artifact_layout'
require 'uu/os/artifact/artifact_reference_entry'
require 'uu/os/artifact/artifact_width_type'
require 'uu/os/artifact/artifact_presentation_attributes'
require 'uu/os/artifact/artifact_set_presentation_attributes'

module UU
  module OS
    # Artifact service.
    module Artifact
      extend self

      # Service path
      PATH = 'ues/core/artifact/UESArtifact'

      # Creates an artifact of class Artifact. Command creates new artifact in specified location.
      # Meta artifact can be specified in ArtifactCreate Object, otherwise is used default meta artifact
      # for territory. Competent role for artifact will be selected as the most suitable according to
      # specified location and meta artifact (executive/authorized role with connected interface),
      # or can be also specified in ArtifactCreate Object.
      #
      # @param [String, UU::OS::UESURI] location_uri  URI of folder/org. unit/meta model where artifact will be created.
      # @param [UU::OS::Artifact::ArtifactCreate] artifact Specific options.
      # @option artifact [String] :code Code of the new artifact.
      # @option artifact [String, UU::OS::UESURI] :competent_role_uri URI of the competent role.
      # @option artifact [String] :description Description of the new artifact.
      # @option artifact [String, UU::OS::UESURI] :icon_uri URI of the artifact's icon. If nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the role icon is removed and the artifact uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option artifact [String, UU::OS::UESURI] :meta_artifact_uri URI of the meta artifact.
      # @option artifact [String] :name Name of the new artifact.
      # @option artifact [String, UU::OS::Artifact::SecurityLevel] :security_level Security level.
      # @option artifact [Boolean] :create_workflow_async Created asynchronously.
      # @option artifact [Boolean] :dry_run Specifies whether this is only a test run to verify that the object can be created. The default value is false.
      # @option artifact [Boolean] :component_content Defines whether the primarily displayed content of the created Artifact is component content. Default value is false.
      # 
      # @return [UU::OS::UESURI] URI of created artifact.
      #    
      # @example
      #   # Create new artifact.
      #   UU::OS::Artifact.create('ues:TERRITORY:FOLDER', :code => 'NEW_ARTIFACT', :name => 'New Artifact')
      #  
      #   # Verifying that new artifact can be created.
      #   UU::OS::Artifact.create('ues:TERRITORY:FOLDER', :code => 'NEW_ARTIFACT', :name => 'New Artifact', :dryRun => true) 
      def create(location_uri, artifact = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = Artifact::ArtifactCreate.new(artifact).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          # res is nil if dry_run was set to true. In that case, return nil (do nothing)
          if !(res.nil? || res.empty?)
            return UU::OS::UESURI.new(res)
          end
        end
      end

      # Gets the attributes of the artifact specified by artifact URI and returns its detail.
      #
      # @param [String, UU::OS::UESURI] artifact_uri URI of required artifact
      #
      # @return [ArtifactAttributes] Object with attributes of artifact
      #
      # @example
      #   # Get artifact attributes.
      #   artifactAttributes = UU::OS::Artifact.get_attributes('ues:UNI:ARTIFACT')
      def get_attributes(artifact_uri)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', artifact_uri)
          return Artifact::ArtifactAttributes.new(res)
        end
      end

      # Command for setting attributes of artifact. Command can't modify references to
      # object like storage, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] artifact_uri  URI of required artifact.
      # @param [ArtifactSetAttributes] artifact Object containing attributes for updating the artifact.
      # @option artifact [String] :code Code of the artifact.
      # @option artifact [String] :description Description of the artifact.
      # @option artifact [String, UU::OS::UESURI] :icon_uri URI of the artifact's icon. If nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:") is set, the role icon is removed and the artifact uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option artifact [String] :name Name of the artifact.
      # @option artifact [String, UU::OS::Artifact::SecurityLevel] :security_level Security level of the artifact.
      # @option artifact [Boolean] :component_content Defines whether the primarily displayed content of the created Artifact is component content. If attribute is not set, the value is not changed.
      #      
      # @return [UU::OS::UESURI] URI of artifact
      #
      # @example
      #   # Set Artifact attributes - type, name, code, description and security level
      #   UU::OS::Artifact.set_attributes('ues:UNI:NEWS',:code=>'ARTIFACT2',:description=>'Some sample description', :name=>'Artifact name', :security_level=>UU::OS::Artifact::SecurityLevel::INTERNAL_USE)
      #
      #   # Set Artifact attributes change only icon uri.
      #   UU::OS::Artifact.set_attributes('ues:UNI:NEWS', :icon_uri=>'ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_077}')
      def set_attributes(artifact_uri, artifact = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactSetAttributes.new(artifact).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Moves an artifact to the new location (folder, organizational unit, meta mode
      # dictionary) in the same territory. It optionally leaves a shortcut at the original location.
      #
      # @param [String, UU::OS::UESURI] artifact_uri  UESURI of the artifact to be moved
      # @param [ArtifactMove] new_location Object containing details of the moving
      # @option new_location [Boolean] :keep_shortcut Specifies if a new shortcut pointing to the artifactUri should be created at the original location.
      # @option new_location [String, UU::OS::UESURI] :location_uri UESURI of a location (folder/organizational unit, meta model) where to the artifact will be moved.
      #
      # @example
      #   # Move artifact to new location without keeping shortcut in original location.
      #   UU::OS::Artifact.move('ues:UNI:ARTIFACT',:keep_shortcut=>false, :location_uri=>'ues:UNI:NEWLOCATION')
      def move(artifact_uri, new_location  = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactMove.new(new_location).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('move', artifact_uri, payload)
        end
      end

      # Deletes the specified artifact from the system.
      # The artifact is deleted even if it contains a locked sheet or attachment.
      # The artifact cannot be deleted when:
      #   - the artifact's workflow is not created
      #   - a property contains a reference to this artifact
      #   - a time sheet exists with a reference to this artifact
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact which is about to be deleted.
      # 
      # @example
      #   # Delete artifact with specified URI.
      #   UU::OS::Artifact.delete('ues:TERRITORY:ARTIFACT')
      def delete(artifact_uri)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('delete', artifact_uri)
        end
      end

      # Sets a new competent role for an artifact (asynchronously).
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact whose competence is about to be changed.
      # @param [ArtifactSetCompetentRole] new_competency Object containing attributes of the competence change
      # @option new_competency [String, UU::OS::UESURI] :competent_role_uri UESURI of the new competent role for the artifact.
      # 
      # @example
      #   # Set artifact competent role.
      #   UU::OS::Artifact.set_competent_role('ues:UNI:ARTIFACT', :competent_role_uri=>'ues:UNI:ROLEA')
      def set_competent_role(artifact_uri, new_competency = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactSetCompetentRole.new(new_competency).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setCompetentRole', artifact_uri, payload)
        end
      end

      # Sets a new state of the specified artifact. The current state of an artifact
      # can be retrieved by get_attributes command. Transition to the new state is driven by an activity
      # on the artifact workflow. The activity itself sets the artifact state. There are three ways to
      # specify this activity:
      #
      # 1. The easiest way is to specify the required artifact state from the enumeration
      # ArtifactStateType. In this scenario, an appropriate activity template is found (an activity
      # transforming the artifact to the required state) and it is used the same way as in the third
      # scenario. If there is no such an activity template, the command fails. If there are a few
      # such activities, then the first one is used (activity templates are ordered by name and code
      # for this purpose).
      #
      # 2. By the same pattern, it is possible to specify UESURI of the artifact state template we want
      # to set. The advantage compared to the first scenario is that we can also set some specific
      # custom states that are not included in the enumeration UESArtifactStateType without knowing
      # the activity template that sets it.
      #
      # 3. The UESURI of the activity template can be also specified directly. This activity has to
      # contain an action to change the artifact state on its exit activity state. The activity is
      # created and it changes the artifact state. If the activity contains an action with an attached
      # script then the script is run first asynchronously and the artifact state is set after the script
      # finishes successfully (therefore the artifact state can be actually set later, e.g. two minutes
      # after set_state command finishes).
      #
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact which state is about to be changed.
      # @param [ArtifactSetState] state Object containing details of the state change.      
      # @option state [String, UU::OS::UESURI] :state_template_uri UESURI of an artifact state template. If there's no activity for setting this state, the command fails. On the contrary, if there are several such activities, the command will pick the first of them. If it is not specified, either activity_template_uri or type is used to determine the new state. Only one of these three parameters can be set to a non-nil value.
      # @option state [String] :comment The artifact state comment. If it is not specified, the comment will be left empty.
      # @option state [String, UU::OS::Artifact::ArtifactStateType] :type The new state of the artifact. If it is not specified, either state_template_uri or activity_template_uri is used to determine the new state. Only one of these three parameters can be set to a non-nil value.
      # 
      # @example
      #   # Set artifact state.
      #   UU::OS::Artifact.set_state('ues:UNI:ARTIFACT',:state_template_uri=>'ues:UNI:ARTIFACT:ues_COMPLETED',:comment=>'Some sample comment') 
      def set_state(artifact_uri, state = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactSetState.new(state).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setState', artifact_uri, payload)
        end
      end

      # Returns a list of all references from the specified artifact and / or all references to the specified artifact. The
      # command does not return all references, only a sublist. The list of returned references can be filtered by the name and code of the
      # artifact, by the meta artifact uri of the artifact, by the type of the reference, direction and the reference count. The list is sorted
      # by the name, code and type by default, or it can be sorted arbitrarily by any of reference attributes usable for the filtering or these
      # attributes combinations.
      # 
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact to which and/or from which the references are listed.
      # @param [ArtifactGetReferenceList] criteria Object containing criteria for listing references. Available parameters are: {ReferenceEntry#name name}, {ReferenceEntry#code code}, {ReferenceEntry#entity_type_uri entity_type_uri}, {ReferenceEntry#type type} : {ReferenceType ReferenceType}, {ReferenceEntry#direction direction} : {ReferenceDirection ReferenceDirection} and {ReferenceEntry#count count}.
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @option criteria [String] :query The <b>UES query</b> string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there. If nil is set, no filtering is applied and entries are ordered by name of referenced artifact (and code and type as the second and the third criteria).
      #
      # @return [UU::OS::REST::ResultList<Artifact::ArtifactGetReferenceList, Artifact::ReferenceEntry>] List of Objects representing references to and/or from the artifact_uri.
      # 
      # @example
      #   # Get reference list for specified artifact without criteria.
      #   UU::OS::Artifact.get_reference_list('ues:UNI:TESTARTIFACT')
      #
      #   # Get reference list with query.
      #   UU::OS::Artifact.get_reference_list('ues:UNI:TESTARTIFACT', :query=>'direction = in ORDER BY entityTypeUri DESC')
      def get_reference_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        dto = UU::OS::Artifact::ArtifactGetReferenceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index) 
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getReferenceList', artifact_uri)
          return UU::OS::REST::ResultList.new(Artifact::ArtifactGetReferenceList, Artifact::ReferenceEntry, res)
        end
      end

      # Sets an artifact presentation settings. The current presentation settings of an artifact can be retrieved by getAttributes
      # command. Artifact presentation settings consists of width. Changing the width of the artifact changes the width of the content part of
      # the artifact. Artifact width is used as default width for all artifact sheets and in visual use cases over artifact and its aggregated
      # objects.
      # 
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact which layout is about to be changed.
      # @param [ArtifactSetVisualPresentation] presentation Object containing presentation settings.
      # @option presentation [String, UU::OS::Artifact::ArtifactLayout] :layout Layout of the artifact. If nil is set, the layout is not modified.
      # @option presentation [String, UU::OS::Artifact::ArtifactArtifactWidthType] :width Width of the artifact. If nil is set, the width is not modified.
      # @deprecated
      def set_visual_presentation(artifact_uri, presentation = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactSetVisualPresentation.new(presentation).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setVisualPresentation', artifact_uri, payload)
        end
      end

      # Configures presentation settings of an artifact. Artifact presentation settings consist of width, layout artifact UESURI, header
      # visibility, toolbar visibility, and artifact display mode. All of these attributes control presentation of the artifact. When viewing
      # an artifact, presentation settings are looked up in the following order: User settings, artifact presentation settings, meta artifact
      # template presentation settings, and layout artifact, if it is configured, or default system settings otherwise.
      #
      # @param [String, UU::OS::UESURI] artifact_uri  URI of the artifact whose presentation settings are to be changed.
      # @param [ArtifactSetPresentationAttributes] presentation_attributes Object containing artifact presentation settings.
      # @option presentation_attributes [String, UU::OS::UESURI] :layout_artifact_uri UESURI referencing a layout artifact. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactWidthType] :artifact_width Width of the artifact. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactHeaderDisplayMode] :header_display_mode Header type. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactControlBarDisplayMode] :control_bar_display_mode Show/hide control bar. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactCommentsDisplayMode] :comment_display_mode Show/hide comments. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactDisplayMode] :artifact_display_mode Show simple or advanced context menu. Never nil.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactVisualStyle] :visual_style Represents visual style.
      # @option presentation_attributes [String, UU::OS::Artifact::ArtifactNavigationAreaDisplayMode] :navigation_area_display_mode Show/hide navigation area when displaying sheet content. Navigation area contains navigation bar, header, heading and control bar.
      #
      # @return [UU::OS::UESURI] URI of artifact
      #
      # @example
      #   # Set presentation attributes of an artifact
      #   UU::OS::Artifact.set_presentation_attributes('ues:TERRITORY:ARTIFACT', :layout_artifact_uri => 'ues:TERRITORY:ARTIFACT2',
      #   :artifact_width => UU::OS::Artifact::ArtifactWidthType::DEFAULT,
      #   :header_display_mode => UU::OS::Artifact::ArtifactHeaderDisplayMode::HIDDEN,
      #   :comment_display_mode => UU::OS::Artifact::ArtifactCommentsDisplayMode::VISIBLE, :visual_style => UU::OS::Artifact::ArtifactVisualStyle::V2)
      def set_presentation_attributes(artifact_uri, presentation_attributes = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactSetPresentationAttributes.new(presentation_attributes).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setPresentationAttributes', artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Retrieves presentation settings of an artifact. Artifact presentation settings consist of width, layout artifact UESURI, header
      # visibility, toolbar visibility, and artifact display mode. All of these attributes control presentation of the artifact. When viewing
      # an artifact, presentation settings are looked up in the following order: User settings, artifact presentation settings, meta artifact
      # template presentation settings, and layout artifact, if it is configured, or default system settings otherwise.
      #                                                                                                                                                                                                                                                                                                                                                                                                                                                                          #
      # @param [String, UU::OS::UESURI] artifact_uri URI of the artifact whose presentation settings are to be retrieved.
      # @return [ArtifactPresentationAttributes] Object with presentation attributes of the artifact
      #
      # @example
      #   # Get presentation attributes of an artifact
      #   UU::OS::Artifact.get_presentation_attributes('ues:TERRITORY:ARTIFACT')
      def get_presentation_attributes(artifact_uri)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getPresentationAttributes', artifact_uri)
          return Artifact::ArtifactPresentationAttributes.new(res)
        end
      end

      # Exports an artifact to the XML file, which is saved to the export storage. The export is an asynchronous process. <br>
      # It is possible to wait for the end of the process. More information in {UU::OS::Env::Process} and in {UU::OS::REST::Future}.
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact to export.
      #
      # @return [UU::OS::UESURI] UESURI of the export process.
      #
      # @example
      #   # Start exporting an artifact.
      #   process_uri = UU::OS::Artifact.export('ues:TERRITORY:ARTIFACT')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      def export(artifact_uri)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        payload = UU::OS::Artifact::ArtifactExport.new(nil).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('export', artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a XML file generated by the {#export export} command.
      #
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of the artifact we want to get the exported XML file from.
      # @param [UU::OS::Artifact::ArtifactGetExportData] artifact Specific options.
      # @option artifact [String, UU::OS::Artifact::ExportMimeType] :requested_mime_type MIME type of the returned value. ZIP is returned by default.
      #
      # @return [UU::OS::REST::BinaryValue] Data file of an exported artifact (XML or ZIP containing XML).
      #
      # @example
      #   # Start exporting an artifact
      #   process_uri = UU::OS::Artifact.export('ues:TERRITORY:ARTIFACT')
      #
      #   # Wait for the end of the process.
      #   export_uri = UU::OS::REST::Future.new(process_uri).get(nil, 600)
      #
      #   # Get exported XML file.
      #   exported_data = UU::OS::Artifact.get_export_data(export_uri, :requested_mime_type => 'application/xml')
      def get_export_data(artifact_uri, artifact = nil)
        svc = UU::OS::REST::RemoteClient.new(Artifact)
        dto = UU::OS::Artifact::ArtifactGetExportData.new(artifact)
        if dto.requested_mime_type
          svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        end
        UU::OS::QoS::QoSHandler.auto_retry do
          return svc.get_binary('getExportData', artifact_uri)
        end
      end

    end
  end
end
