require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/cmd'
require 'uu/os/meta_model/artifact_interface/artifact_interface_add_use_case'
require 'uu/os/meta_model/artifact_interface/artifact_interface_attributes'
require 'uu/os/meta_model/artifact_interface/artifact_interface_create'
require 'uu/os/meta_model/artifact_interface/artifact_interface_get_artifact_interface_list'
require 'uu/os/meta_model/artifact_interface/artifact_interface_get_use_case_list'
require 'uu/os/meta_model/artifact_interface/artifact_interface_remove_use_case'
require 'uu/os/meta_model/artifact_interface/artifact_interface_set_attributes'
require 'uu/os/meta_model/artifact_interface/artifact_interface_use_case_attributes'



# Module Unicorn Universe.
module UU
  # Module Operating System.
  module OS
    # Artifact interface service.
    module MetaModel

      class ArtifactInterface

        # Creates a new instance of +ArtifactInterface+.
        #
        # @param session [UU::OS::Security::Session] Session to use for remote calls.
        def initialize(session)
          options = {use_legacy_config: true, append_routing_data: false}
          @cmd = UU::OS::CMD::CommandClient.new("uu/os/meta_model/cmd", session, options)
        end

        # Returns the UESURI of the newly created artifact interface.
        #
        # @param [String, UU::OS::UESURI] meta_artifact_uri UESURI of the meta object for which artifact interface will be created.
        # @param [UU::OS::MetaModel::ArtifactInterfaceCreate] artifact_interface Object containing the attributes of the artifact interface.
        # @option artifact_interface [String] :name Name of the new artifact interface. Must not be nil.
        # @option artifact_interface [String] :code Code of the new artifact interface. Must be unique for the template object. When nil set, new code is generated automatically from the sequence. It cannot be changed after the artifact creation.
        # @option artifact_interface [String] :description Description of the new artifact interface.
        # @option artifact_interface [Boolean] :grant_all_from_territory (false) Flag indicating whether the new artifact interface is automatically connected to users with access to the territory. Default value is false.
        #
        # @return [UU::OS::UESURI] The UESURI of the newly created artifact interface.
        #
        # @example
        #   # Create a new artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.create('ues:TERRITORY:META_ARTIFACT',
        #       :name => 'New Interface',
        #       :code => 'NEW_INTERFACE',
        #       :description => 'New artifact interface',
        #       :grant_all_from_territory => false
        #   )
        def create(meta_artifact_uri, artifact_interface = nil)
          payload = wrap_params(UU::OS::MetaModel::ArtifactInterfaceCreate.new(artifact_interface))
          res = @cmd.invoke("ArtifactInterface/create", meta_artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end

        # {#create} convenient self-instantiating shortcut
        def self.create(meta_artifact_uri, artifact_interface = nil)
          session = UU::OS::Security::Session.current_session
          self.new(session).create(meta_artifact_uri, artifact_interface)
        end

        # Returns an Object containing the artifact interface attributes.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface whose attributes will be returned.
        #
        # @return [UU::OS::MetaModel::ArtifactInterfaceAttributes] An object containing the artifact interface attributes.
        #   * :uri ({UU::OS::UESURI UU::OS::UESURI}) - Normalized UESURI of the artifact interface.
        #   * :name (<code>String</code>) - Name of the artifact interface.
        #   * :code (<code>String</code>) - Code of the artifact interface.
        #   * :description (<code>String</code>) - Description of the artifact interface.
        #   * :grant_all_from_territory (<code>Boolean</code>) - Flag indicating whether the new artifact interface is automatically connected to users with access to the territory.
        #   * :creation_stamp (<code>String</code>) - Time when the artifact interface was created.
        #   * :modification_stamp (<code>String</code>) - Time when the artifact interface was modified.
        #
        # @example
        #   # Get attributes of an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.get_attributes('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE')
        def get_attributes(artifact_interface_uri)
          res = @cmd.invoke('ArtifactInterface/getAttributes', artifact_interface_uri) #, invocation_method: 'get')
          return UU::OS::MetaModel::ArtifactInterfaceAttributes.new(res)
        end

        # {#get_attributes} convenient self-instantiating shortcut
        def self.get_attributes(artifact_interface_uri)
          session = UU::OS::Security::Session.current_session
          self.new(session).get_attributes(artifact_interface_uri)
        end

        # Returns UESURI of the updated artifact interface.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface whose attributes will be modified.
        # @param [UU::OS::MetaModel::ArtifactInterfaceSetAttributes] artifact_interface Object containing basic attributes of the artifact interface that will be updated.
        # @option artifact_interface [String] :name New name of the artifact interface. When nil is set, the attribute will not be modified. Must not be blank.
        # @option artifact_interface [String] :description New description of the artifact interface. When nil is set, the attribute will not be modified.
        # @option artifact_interface [Boolean] :grant_all_from_territory Flag indicating whether the new artifact interface is automatically connected to users with access to the territory. When nil is set, the attribute will not be modified.
        #
        # @return [UU::OS::UESURI] UESURI of the updated artifact interface.
        #
        # @example
        #   # Modify a few attributes of an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.set_attributes('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE',
        #       :name => 'New name',
        #       :description => 'New artifact interface description',
        #       :grant_all_from_territory => true
        #   )
        def set_attributes(artifact_interface_uri, artifact_interface = nil)
          payload = wrap_params(UU::OS::MetaModel::ArtifactInterfaceSetAttributes.new(artifact_interface))
          res = @cmd.invoke('ArtifactInterface/setAttributes', artifact_interface_uri, payload)
          return UU::OS::UESURI.new(res)
        end

        # {#set_attributes} convenient self-instantiating shortcut
        def self.set_attributes(artifact_interface_uri, artifact_interface = nil)
          session = UU::OS::Security::Session.current_session
          self.new(session).set_attributes(artifact_interface_uri, artifact_interface)
        end

        # Deletes an artifact interface.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface which is about to be deleted.
        #
        # @example
        #   # Delete an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.delete('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE')
        def delete(artifact_interface_uri)
          @cmd.invoke("ArtifactInterface/delete", artifact_interface_uri)
        end

        # {#delete} convenient self-instantiating shortcut
        def self.delete(artifact_interface_uri)
          session = UU::OS::Security::Session.current_session
          self.new(session).delete(artifact_interface_uri)
        end

        # Retrieves the list of artifact interfaces connected to the given meta artifact. The list is sorted by name of the artifact
        # interface by default.
        #
        # @param [String, UU::OS::UESURI] meta_artifact_uri UESURI of the meta artifact for which the artifact interface list will be returned.
        # @param [UU::OS::MetaModel::ArtifactInterfaceGetArtifactInterfaceList] criteria DTO containing criteria for listing artifact interfaces.
        # @option criteria [String] :query Query for filtering and ordering of the result list. See uuQuery 2.0 documentation for further details. It is possible to filter the result list by name and code.
        #
        # @see https://plus4u.net/ues/sesm?&SessFree=ues:UNI-BT:UU.OS%2FQUERY-2.0 uuQuery 2.0 Documentation
        #
        # @return [UU::OS::REST::ResultList<UU::OS::MetaModel::ArtifactInterfaceGetArtifactInterfaceList, UU::OS::MetaModel::ArtifactInterfaceAttributes>] A list of Objects containing artifact interface attributes. See {#get_attributes get_attributes}.
        #
        # @example
        #   # Get list of artifact interfaces whose names end with "Interface". Order the result list by name and code.
        #   UU::OS::MetaModel::ArtifactInterface.get_artifact_interface_list('ues:TERRITORY:META_ARTIFACT',
        #       :query => 'name="%Interface" ORDER BY name,code'
        def get_artifact_interface_list(meta_artifact_uri, criteria = nil)
          payload = wrap_params(UU::OS::MetaModel::ArtifactInterfaceGetArtifactInterfaceList.new(criteria))
          res = @cmd.invoke('ArtifactInterface/getArtifactInterfaceList', meta_artifact_uri, payload) #, invocation_method: 'get')
          ret = {:total_size => res.size, :page_entries => res.to_a}
          return UU::OS::REST::ResultList.new(UU::OS::MetaModel::ArtifactInterfaceGetArtifactInterfaceList, UU::OS::MetaModel::ArtifactInterfaceAttributes, ret)
        end

        # {#get_artifact_interface_list} convenient self-instantiating shortcut
        def self.get_artifact_interface_list(meta_artifact_uri, criteria = nil)
          session = UU::OS::Security::Session.current_session
          self.new(session).get_artifact_interface_list(meta_artifact_uri, criteria)
        end

        # Connects the selected use case into the selected artifact interface.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface whose data will be modified.
        # @param [UU::OS::MetaModel::ArtifactInterfaceAddUseCase] use_case Object containing the use case code which is to be connected.
        # @option use_case [String] :use_case_code Code of the use case which is to be connected to the artifact interface. Must not be nil.
        #
        # @example
        #   # Connect a use case to an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.add_use_case('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE',
        #       :use_case_code => 'UU.OS/CONTENT/SHEET/GET_SHEET_LIST'
        #   )
        def add_use_case(artifact_interface_uri, use_case = nil)
          payload = wrap_params(UU::OS::MetaModel::ArtifactInterfaceAddUseCase.new(use_case))
          @cmd.invoke('ArtifactInterface/addUseCase', artifact_interface_uri, payload)
        end

        # {#add_use_case} convenient self-instantiating shortcut
        def self.add_use_case(artifact_interface_uri, use_case = nil)
          session = UU::OS::Security::Session.current_session
          self.new(session).add_use_case(artifact_interface_uri, use_case)
        end

        # Removes the selected use case from the selected artifact interface.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface whose data will be modified.
        # @param [UU::OS::MetaModel::ArtifactInterfaceRemoveUseCase] use_case Object containing the use case code which is to be removed.
        # @option use_case [String] :use_case_code Code of the use case which is to be removed from the artifact interface. Must not be nil.
        #
        # @example
        #   # Disconnect a use case from an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.remove_use_case('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE',
        #       :use_case_code => 'UU.OS/CONTENT/SHEET/GET_SHEET_LIST'
        #   )
        def remove_use_case(artifact_interface_uri, use_case)
          payload = wrap_params(UU::OS::MetaModel::ArtifactInterfaceRemoveUseCase.new(use_case))
          @cmd.invoke('ArtifactInterface/removeUseCase', artifact_interface_uri, payload)
        end

        # {#remove_use_case} convenient self-instantiating shortcut
        def self.remove_use_case(artifact_interface_uri, use_case)
          session = UU::OS::Security::Session.current_session
          self.new(session).remove_use_case(artifact_interface_uri, use_case)
        end

        # Retrieves the code list of use cases connected to the given artifact interface in json format.
        #
        # @param [String, UU::OS::UESURI] artifact_interface_uri UESURI of the artifact interface for which the data will be returned.
        #
        # @return [UU::OS::REST::ResultList<UU::OS::MetaModel::ArtifactInterfaceGetUseCaseList, UU::OS::MetaModel::ArtifactInterfaceUseCaseAttributes>] A list of connected use cases of the given artifact interface.
        #
        # @example
        #   # Get list of use cases connected to an artifact interface.
        #   UU::OS::MetaModel::ArtifactInterface.get_use_case_list('ues:TERRITORY:META_ARTIFACT:ARTIFACT_INTERFACE')
        def get_use_case_list(artifact_interface_uri)
          res = @cmd.invoke('ArtifactInterface/getUseCaseList', artifact_interface_uri) #, invocation_method: 'get')
          ret = {:total_size => res.size, :page_entries => res.to_a}
          return UU::OS::REST::ResultList.new(UU::OS::MetaModel::ArtifactInterfaceGetUseCaseList, UU::OS::MetaModel::ArtifactInterfaceUseCaseAttributes, ret)
        end

        # {#get_use_case_list} convenient self-instantiating shortcut
        def self.get_use_case_list(artifact_interface_uri)
          session = UU::OS::Security::Session.current_session
          self.new(session).get_use_case_list(artifact_interface_uri)
        end

        private

        def wrap_params(params)
          parameters = Hash.new
          parameters[:parameters] = params
          return parameters
        end

      end

    end
  end
end
